package com.aliyun.iotx.linkvisual.app;

import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.aliyun.iotx.linkvisual.media.video.PlayerException;
import com.aliyun.iotx.linkvisual.media.video.beans.PlayerState;
import com.aliyun.iotx.linkvisual.media.video.player.ExoHlsPlayer;
import com.aliyun.iotx.linkvisual.media.video.listener.OnErrorListener;
import com.aliyun.iotx.linkvisual.media.video.listener.OnPlayerStateChangedListener;
import com.google.android.exoplayer2.ui.SimpleExoPlayerView;
import org.androidannotations.annotations.AfterViews;
import org.androidannotations.annotations.EActivity;
import org.androidannotations.annotations.Extra;
import org.androidannotations.annotations.UiThread;
import org.androidannotations.annotations.ViewById;



@EActivity(R.layout.activity_hls_player)
public class HLSPlayerActivity extends AppCompatActivity  {

    @Extra("url")
    String videoSource;

    public static final String VIDEO_SOURCE = "http://devimages.apple.com.edgekey.net/streaming/examples/bipbop_4x3/gear3/prog_index.m3u8";

    @ViewById(R.id.hls_player_view)
    SimpleExoPlayerView simpleExoPlayerView;

    @ViewById(R.id.hls_progressbar)
    ProgressBar progressBar;

    private ExoHlsPlayer player;

    @AfterViews
    public void initPlayer(){
        player = new ExoHlsPlayer(getApplicationContext());
        simpleExoPlayerView.setPlayer(player.getExoPlayer());
        simpleExoPlayerView.requestFocus();
        player.setOnErrorListener(new OnErrorListener() {
            @Override
            public void onError(PlayerException exception) {
                setProgressBarVisibility(progressBar,View.GONE);
                makeToast("errorcode: " + exception.getCode() + "\n" + exception.getMessage());
            }
        });
        player.setOnPlayerStateChangedListener(new OnPlayerStateChangedListener() {
            @Override
            public void onPlayerStateChange(int playerState) {
                switch (playerState) {
                    case PlayerState.STATE_BUFFERING:
                        setProgressBarVisibility(progressBar,View.VISIBLE);
                        break;
                    case PlayerState.STATE_IDLE:
                        break;
                    case PlayerState.STATE_READY:
                        setProgressBarVisibility(progressBar,View.GONE);
                        break;
                    case PlayerState.STATE_ENDED:
                        break;
                    default:
                        break;
                }
            }
        });

        if(TextUtils.isEmpty(videoSource)) {
            videoSource = VIDEO_SOURCE;
        }

        player.setDataSource(videoSource);
        player.prepare();
        player.start();
    }

    @UiThread
    public void setProgressBarVisibility(ProgressBar progressBar, int visibility) {
        progressBar.setVisibility(visibility);
    }

    @UiThread
    public void makeToast(String text){
        Toast.makeText(this, text, Toast.LENGTH_SHORT).show();
    }

    @Override
    protected void onPause() {
        player.stop();
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        player.release();
        super.onDestroy();
    }
}