package com.aliyun.iotx.linkvisual.app;

import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.media.MediaCodec;
import android.opengl.GLSurfaceView;
import android.support.constraint.ConstraintLayout;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.aliyun.iotx.linkvisual.media.video.PlayerException;
import com.aliyun.iotx.linkvisual.media.video.beans.PlayerState;
import com.aliyun.iotx.linkvisual.media.video.listener.OnErrorListener;
import com.aliyun.iotx.linkvisual.media.video.listener.OnPlayerStateChangedListener;
import com.aliyun.iotx.linkvisual.media.video.listener.OnPreparedListener;
import com.aliyun.iotx.linkvisual.media.video.player.LivePlayer;
import com.squareup.leakcanary.RefWatcher;

import org.androidannotations.annotations.AfterViews;
import org.androidannotations.annotations.Background;
import org.androidannotations.annotations.CheckedChange;
import org.androidannotations.annotations.EActivity;
import org.androidannotations.annotations.Extra;
import org.androidannotations.annotations.UiThread;
import org.androidannotations.annotations.ViewById;
import org.json.JSONException;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;

import static com.aliyun.iotx.linkvisual.app.MainActivity.verifyStoragePermissions;
import static java.util.concurrent.Executors.newScheduledThreadPool;
import static java.util.concurrent.TimeUnit.SECONDS;

@EActivity(R.layout.activity_live)
public class LivePlayerActivity extends AppCompatActivity {

    public static final String TAG = "VodPlayerActivity";

    @ViewById(R.id.player_glsurfaceview)
    GLSurfaceView playerGLSurfaceView;

    @ViewById(R.id.player_progressbar)
    ProgressBar playerProgressBar;

    @Extra("url")
    String videoSource;

    ScheduledExecutorService scheduledExecutorService = newScheduledThreadPool(1);
    ScheduledFuture<?> playInfoUpdateHandle;

    private LivePlayer player;

    public static final String VIDEO_SOURCE = "rtmp://live.hkstv.hk.lxdns.com/live/hks2";

    @AfterViews
    public void initPlayers() {
        player = new LivePlayer();
        player.setSurfaceView(playerGLSurfaceView);
        player.setOnPlayerStateChangedListener(new OnPlayerStateChangedListener() {
            @Override
            public void onPlayerStateChange(int playerState) {
                Log.d(TAG, "play state= " + playerState);
                switch (playerState) {
                    case PlayerState.STATE_BUFFERING:
                        playerProgressBar.setVisibility(View.VISIBLE);
                        break;
                    case PlayerState.STATE_IDLE:
                        break;
                    case PlayerState.STATE_READY:
                        playerProgressBar.setVisibility(View.GONE);
                        updatePlayInfo();
                        try {
                            makeToast(player.getStatisticsInfo().toString(4));
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                        break;
                    case PlayerState.STATE_ENDED:
                        playerProgressBar.setVisibility(View.GONE);
                        break;
                    default:
                        break;
                }
            }
        });
        player.setOnErrorListener(new OnErrorListener() {
            @Override
            public void onError(PlayerException exception) {
                playerProgressBar.setVisibility(View.GONE);
                makeToast("errorcode: " + exception.getCode() + "\n" + exception.getMessage());
            }
        });
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        if (newConfig.orientation == ActivityInfo.SCREEN_ORIENTATION_USER) {
            ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) playerGLSurfaceView.getLayoutParams();
            layoutParams.height = ConstraintLayout.LayoutParams.MATCH_PARENT;
            playerGLSurfaceView.setLayoutParams(layoutParams);
            hideSystemUI();
        } else {
            ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) playerGLSurfaceView.getLayoutParams();
            layoutParams.height = (int) (220 * getResources().getDisplayMetrics().density);
            playerGLSurfaceView.setLayoutParams(layoutParams);
            showSystemUI();
        }
        super.onConfigurationChanged(newConfig);
    }

    @Override
    protected void onResume() {
        super.onResume();
        verifyStoragePermissions(this);
        playerGLSurfaceView.onResume();
    }

    @Override
    protected void onPause() {
        player.stop();
        super.onPause();
        playerGLSurfaceView.onPause();
    }

    @Override
    protected void onDestroy() {
        player.release();
        if(playInfoUpdateHandle != null){
            playInfoUpdateHandle.cancel(true);
        }
        super.onDestroy();
        RefWatcher refWatcher = MyAppication.getRefWatcher(this);
        refWatcher.watch(this);
    }

    @Background(id = "play")
    public void onPlayClicked(View view) {
        if(TextUtils.isEmpty(videoSource)) {
             videoSource = VIDEO_SOURCE;
        }

        player.setDataSource(videoSource, false, new byte[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ':', ';', '<', '=', '>', '?'}, new byte[]{'1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '0', '0', '0', '0', '0', '0'});
        player.setOnPreparedListener(new OnPreparedListener() {
            @Override
            public void onPrepared() {
                player.start();
            }
        });
        player.prepare();
    }

    @Background(id = "play")
    public void onStopClicked(View view) {
        player.stop();
    }

    public void onSnapshotClicked(View view) {
        Bitmap bitmap = player.snapShot();
        if(bitmap == null){
            makeToast("无法截图，当前无画面");
            return;
        }
        SnapshotPreviewDialog snapshotPreviewDialog = new SnapshotPreviewDialog(this);
        snapshotPreviewDialog.show();
        snapshotPreviewDialog.setImageBitmap(bitmap);
    }

    @CheckedChange(R.id.full_screen_switch)
    public void fullScreenSwitchChecked(CompoundButton button, boolean isChecked) {
        if (isChecked) {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        } else {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        }
    }

    @CheckedChange(R.id.sacling_mode_switch)
    public void scalingModeSwitchChecked(CompoundButton button, boolean isChecked){
        if(isChecked){
            player.setVideoScalingMode(MediaCodec.VIDEO_SCALING_MODE_SCALE_TO_FIT);
        }else{
            player.setVideoScalingMode(MediaCodec.VIDEO_SCALING_MODE_SCALE_TO_FIT_WITH_CROPPING);
        }
    }

    final Runnable updatePlayInfoTask = new Runnable() {
        @Override
        public void run() {
            updatePlayInfo();
        }
    };

    @UiThread
    public void updatePlayInfo() {
        if (playInfoUpdateHandle == null) {
            playInfoUpdateHandle = scheduledExecutorService.scheduleAtFixedRate(updatePlayInfoTask, 1, 1, SECONDS);
        }
        Log.d(TAG, player.getCurrentPlayInfo().toString());
    }

    @UiThread
    public void makeToast(String text) {
        Toast.makeText(this, text, Toast.LENGTH_SHORT).show();
    }

    private void hideSystemUI() {
        View decorView = getWindow().getDecorView();
        decorView.setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_IMMERSIVE
                        // Set the content to appear under the system bars so that the
                        // content doesn't resize when the system bars hide and show.
                        | View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                        // Hide the nav bar and status bar
                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_FULLSCREEN);
    }

    private void showSystemUI() {
        View decorView = getWindow().getDecorView();
        decorView.setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN);
    }

}
