package com.aliyun.iotx.linkvisual.app;

import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.Toast;

import com.github.chrisbanes.photoview.PhotoView;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;


/**
 * @Author: EverettLi.ll
 * @Date: 11/7/18
 * @Description: 视频截图预览窗口
 **/
public class SnapshotPreviewDialog extends Dialog implements View.OnClickListener {

    Bitmap bitmap;

    PhotoView photoView;
    Button closeButton;
    Button saveButton;

    public SnapshotPreviewDialog(@NonNull Context context) {
        super(context);
    }

    public SnapshotPreviewDialog(@NonNull Context context, int themeResId) {
        super(context, themeResId);
    }



    protected SnapshotPreviewDialog(@NonNull Context context, boolean cancelable, @Nullable OnCancelListener cancelListener) {
        super(context, cancelable, cancelListener);
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.snapshot_preview_dialog);
        photoView = findViewById(R.id.photo_view);
        closeButton =findViewById(R.id.close_button);
        saveButton = findViewById(R.id.save_button);
        closeButton.setOnClickListener(this);
        saveButton.setOnClickListener(this);
    }

    public void setImageBitmap(Bitmap bitmap){
        this.bitmap = bitmap;
        photoView.setImageBitmap(bitmap);
    }


    @Override
    public void onClick(View v) {
        if(v == closeButton) {
            if (isShowing()) {
                dismiss();
            }
        }else if(v == saveButton){
            SnapshotPreviewDialog.saveImageToGallery(getContext(), bitmap);
            Toast.makeText(getContext(), "已保存", Toast.LENGTH_SHORT).show();
            dismiss();
        }
    }

    public static void saveImageToGallery(Context context, Bitmap bmp) {
        File appDir = new File(Environment.getExternalStorageDirectory(), "linkvision");
        if (!appDir.exists()) {
            appDir.mkdir();
        }
        String fileName = System.currentTimeMillis() + ".jpg";
        File file = new File(appDir, fileName);
        try {
            FileOutputStream fos = new FileOutputStream(file);
            bmp.compress(Bitmap.CompressFormat.JPEG, 100, fos);
            fos.flush();
            fos.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        try {
            MediaStore.Images.Media.insertImage(context.getContentResolver(),
                    file.getAbsolutePath(), fileName, null);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        context.sendBroadcast(new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.parse("file://" + file.getAbsolutePath())));
    }
}
