package com.aliyun.iotx.linkvisual.app;

import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.media.MediaCodec;
import android.opengl.GLSurfaceView;
import android.support.constraint.ConstraintLayout;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.ProgressBar;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import com.aliyun.iotx.linkvisual.media.video.PlayerException;
import com.aliyun.iotx.linkvisual.media.video.beans.PlayerState;
import com.aliyun.iotx.linkvisual.media.video.listener.OnErrorListener;
import com.aliyun.iotx.linkvisual.media.video.listener.OnPlayerStateChangedListener;
import com.aliyun.iotx.linkvisual.media.video.player.VodPlayer;
import com.squareup.leakcanary.RefWatcher;

import org.androidannotations.annotations.AfterViews;
import org.androidannotations.annotations.Background;
import org.androidannotations.annotations.CheckedChange;
import org.androidannotations.annotations.EActivity;
import org.androidannotations.annotations.Extra;
import org.androidannotations.annotations.SeekBarProgressChange;
import org.androidannotations.annotations.SeekBarTouchStart;
import org.androidannotations.annotations.SeekBarTouchStop;
import org.androidannotations.annotations.UiThread;
import org.androidannotations.annotations.ViewById;

import java.text.SimpleDateFormat;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;

import static com.aliyun.iotx.linkvisual.app.MainActivity.verifyStoragePermissions;
import static java.util.concurrent.Executors.*;
import static java.util.concurrent.TimeUnit.MILLISECONDS;
import static java.util.concurrent.TimeUnit.SECONDS;

@EActivity(R.layout.activity_vod)
public class VodPlayerActivity extends AppCompatActivity {

    public static final String TAG = "VodPlayerActivity";

    @ViewById(R.id.player_glsurfaceview)
    GLSurfaceView playerGLSurfaceView;

    @ViewById(R.id.player_progressbar)
    ProgressBar playerProgressBar;

    @ViewById(R.id.player_seek_bar)
    SeekBar playerSeekBar;

    @ViewById(R.id.duration_textview)
    TextView durationTextView;

    @ViewById(R.id.pause_resume_btn)
    CompoundButton pauseResumeBtn;

    @Extra("url")
    String videoSource;

    ScheduledExecutorService scheduledExecutorService = newScheduledThreadPool(1);
    ScheduledFuture<?> timelineUpdateHandle;
    ScheduledFuture<?> playInfoUpdateHandle;
    final SimpleDateFormat timeLineFormatter = new SimpleDateFormat("mm:ss");

    private VodPlayer player;

    public static final String VIDEO_SOURCE = "rtmp://30.43.88.232/vod?token=1&session=unencrypt_01/success";

    @AfterViews
    public void initPlayers() {
        player = new VodPlayer();
        player.setSurfaceView(playerGLSurfaceView);
        player.setOnPlayerStateChangedListener(new OnPlayerStateChangedListener() {
            @Override
            public void onPlayerStateChange(int playerState) {
                Log.d(TAG, "play state= " + playerState);
                switch (playerState) {
                    case PlayerState.STATE_BUFFERING:
                        showProgress();
                        cancelUpdateTimeline();
                        break;
                    case PlayerState.STATE_IDLE:
                        break;
                    case PlayerState.STATE_READY:
                        dismissProgress();
                        pauseResumeBtn.setChecked(true);
                        updateTimeline();
                        updatePlayInfo();
                        break;
                    case PlayerState.STATE_ENDED:
                        dismissProgress();
                        break;
                    default:
                        break;
                }
            }
        });
        player.setOnErrorListener(new OnErrorListener() {
            @Override
            public void onError(PlayerException exception) {
                dismissProgress();
                makeToast("errorcode: " + exception.getCode() + "\n" + exception.getMessage());
            }
        });
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        if (newConfig.orientation == ActivityInfo.SCREEN_ORIENTATION_USER) {
            ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) playerGLSurfaceView.getLayoutParams();
            layoutParams.height = ConstraintLayout.LayoutParams.MATCH_PARENT;
            playerGLSurfaceView.setLayoutParams(layoutParams);
            hideSystemUI();
        } else {
            ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) playerGLSurfaceView.getLayoutParams();
            layoutParams.height = (int) (220 * getResources().getDisplayMetrics().density);
            playerGLSurfaceView.setLayoutParams(layoutParams);
            showSystemUI();
        }
        super.onConfigurationChanged(newConfig);
    }

    @Override
    protected void onResume() {
        super.onResume();
        verifyStoragePermissions(this);
        playerGLSurfaceView.onResume();
    }

    @Override
    protected void onPause() {
        player.stop();
        super.onPause();
        playerGLSurfaceView.onPause();
    }

    @Override
    protected void onDestroy() {
        player.release();
        cancelUpdateTimeline();
        cancelUpdatePlayInfo();
        super.onDestroy();
        RefWatcher refWatcher = MyAppication.getRefWatcher(this);
        refWatcher.watch(this);
    }

    @Background(id = "play")
    public void onPlayClicked(View view) {
        if (TextUtils.isEmpty(videoSource)) {
            videoSource = VIDEO_SOURCE;
        }

        player.setDataSource(videoSource, false, new byte[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ':', ';', '<', '=', '>', '?'}, new byte[]{'1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '0', '0', '0', '0', '0', '0'});
        player.prepare();
        player.start();
    }

    @Background(id = "play")
    public void onStopClicked(View view) {
        player.stop();
    }

    public void onSnapshotClicked(View view) {
        Bitmap bitmap = player.snapShot();
        if (bitmap == null) {
            makeToast("无法截图，当前无画面");
            return;
        }
        SnapshotPreviewDialog snapshotPreviewDialog = new SnapshotPreviewDialog(this);
        snapshotPreviewDialog.show();
        snapshotPreviewDialog.setImageBitmap(bitmap);
    }

    @CheckedChange(R.id.full_screen_switch)
    public void fullScreenSwitchChecked(CompoundButton button, boolean isChecked) {
        if (isChecked) {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        } else {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        }
    }

    @CheckedChange(R.id.sacling_mode_switch)
    public void scalingModeSwitchChecked(CompoundButton button, boolean isChecked) {
        if (isChecked) {
            player.setVideoScalingMode(MediaCodec.VIDEO_SCALING_MODE_SCALE_TO_FIT);
        } else {
            player.setVideoScalingMode(MediaCodec.VIDEO_SCALING_MODE_SCALE_TO_FIT_WITH_CROPPING);
        }
    }

    @CheckedChange(R.id.pause_resume_btn)
    public void pauseResumeSwitchChecked(CompoundButton button, boolean isChecked) {
        if (isChecked) {
            player.start();
        } else {
            player.pause();
        }
    }

    @SeekBarTouchStart(R.id.player_seek_bar)
    public void onSeekBarTouchStart(SeekBar seekbar) {
        Log.d(TAG, "onSeekBarTouchStart");
        cancelUpdateTimeline();
    }

    @SeekBarTouchStop(R.id.player_seek_bar)
    public void onSeekBarTouchStop(SeekBar seekbar) {
        showProgress();
        int currentProgressInMS = (int) (seekbar.getProgress() * player.getDuration() / playerSeekBar.getMax());
        player.seekTo(currentProgressInMS);
        Log.d(TAG, "onSeekBarTouchStop");
    }

    @SeekBarProgressChange(R.id.player_seek_bar)
    public void onProgressChangeOnSeekBar(SeekBar seekBar, int progress) {
        if (player.getDuration() > 0) {
            int currentProgress = (int) (progress * player.getDuration() / playerSeekBar.getMax());
            durationTextView.setText(timeLineFormatter.format(currentProgress) + "/" + timeLineFormatter.format(player.getDuration()));
        }
    }

    final Runnable updateTimeLineTask = new Runnable() {
        @Override
        public void run() {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if (player.getDuration() <= 0) {
                        durationTextView.setText("-/-");
                        playerSeekBar.setProgress(0);
                    } else {
                        durationTextView.setText(timeLineFormatter.format(player.getCurrentPosition()) + "/" + timeLineFormatter.format(player.getDuration()));
                        playerSeekBar.setProgress((int) (player.getCurrentPosition() * playerSeekBar.getMax() / player.getDuration()));
                    }
                }
            });

        }
    };

    public void updateTimeline() {
        if (timelineUpdateHandle == null) {
            timelineUpdateHandle = scheduledExecutorService.scheduleAtFixedRate(updateTimeLineTask, 500, 100, MILLISECONDS);
        }
    }

    private void cancelUpdateTimeline() {
        if (timelineUpdateHandle != null) {
            timelineUpdateHandle.cancel(true);
            timelineUpdateHandle = null;
        }
    }

    final Runnable updatePlayInfoTask = new Runnable() {
        @Override
        public void run() {
            Log.d(TAG, player.getCurrentPlayInfo().toString());
        }
    };

    public void updatePlayInfo() {
        if (playInfoUpdateHandle == null) {
            playInfoUpdateHandle = scheduledExecutorService.scheduleAtFixedRate(updatePlayInfoTask, 1, 1, SECONDS);
        }
    }

    private void cancelUpdatePlayInfo() {
        if (playInfoUpdateHandle != null) {
            playInfoUpdateHandle.cancel(true);
            playInfoUpdateHandle = null;
        }
    }

    @UiThread
    public void showProgress() {
        playerProgressBar.setVisibility(View.VISIBLE);
    }

    @UiThread
    public void dismissProgress() {
        playerProgressBar.setVisibility(View.GONE);
    }

    @UiThread
    public void makeToast(String text) {
        Toast.makeText(this, text, Toast.LENGTH_SHORT).show();
    }

    private void hideSystemUI() {
        View decorView = getWindow().getDecorView();
        decorView.setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_IMMERSIVE
                        // Set the content to appear under the system bars so that the
                        // content doesn't resize when the system bars hide and show.
                        | View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                        // Hide the nav bar and status bar
                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_FULLSCREEN);
    }

    private void showSystemUI() {
        View decorView = getWindow().getDecorView();
        decorView.setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN);
    }
}
